import { GraphQLField } from "graphql";
import { Args, ResolveFunction, Middleware, FieldType, StringKeyOf, InputField, Removable, KeyValue, ClassType, InputType } from "../..";
import { GQLField } from "./GQLField";
export declare class Field<NameType = string, MetaType = KeyValue> extends GQLField<GraphQLField<any, any, any>, NameType, MetaType> {
    private _args;
    private _doParseArgs;
    private _resolve;
    private _middlewares;
    get args(): Args<any, KeyValue>[];
    get flatArgs(): import("./Arg").Arg<string, KeyValue>[];
    get doParseArgs(): boolean;
    get resolve(): ResolveFunction<any, KeyValue>;
    protected constructor(name: NameType & string, type: FieldType);
    /**
     * Create a new Field for ObjectType and InterfaceType
     */
    static create<NameType = any>(name: StringKeyOf<NameType>, type: FieldType): Field<StringKeyOf<NameType>>;
    static create(name: string, type: FieldType): Field<string>;
    static create<NameType = any>(field: InputField<any>): Field<StringKeyOf<NameType>>;
    static create<NameType = any>(field: Field<any>): Field<StringKeyOf<NameType>>;
    /**
     * Set the middlewares to your field
     * A middleware is executed before your main function
     * @param middlewares The middlewares list
     */
    setMiddlewares(...middlewares: Middleware[]): this;
    /**
     * Add some middlewares to your field
     * A middleware is executed before your main function
     * @param middlewares The middlewares list
     */
    addMiddlewares(...middlewares: Middleware[]): this;
    /**
     * Remove some middlewares from your field
     * @param middlewares The middlewares list
     */
    removeMiddlewares(...middlewares: Removable<Middleware>): this;
    /**
     * If you don't want to parse your aguments objects into the specified class (enabled by default)
     */
    disableArgsParsing(): this;
    /**
     * If you want to parse your aguments objects into the specified class (enabled by default)
     */
    enableArgsParsing(): this;
    /**
     * Set the arguments of your field
     * @param args The aguments
     */
    setArgs(...args: Args[]): this;
    /**
     * Add arguments to your field
     * @param args The aguments
     */
    addArgs(...args: Args[]): this;
    /**
     * Set the resolver function to your field
     * @param args The aguments
     */
    setResolver<ReturnType = any, ArgType = KeyValue>(resolver: ResolveFunction<ReturnType, ArgType>, ...args: Args<ClassType<ArgType>>[]): this;
    /**
     * Copy the field and return a new one
     */
    copy(): Field<NameType>;
    /**
     * Convert your field into a new field type
     * @param to The target type
     */
    convert(to: typeof InputField): InputField<NameType>;
    protected parseArgs(args: (Args | InputType)[], values: KeyValue): {};
    build(): GraphQLField<any, any, any>;
    private resolveField;
}
